#include <gb/gb.h>
#include <gb/drawing.h>
#include <stdio.h>
#include <gb/console.h>
#include <gb/sample.h>

#include "tiles.c"
#include "audio.c"
#include "moveable.c"

/*
 * sprites
 */

#define SPR_PLAYER 1
#define SPR_BULLET_0 2
#define SPR_BULLET_1 3
#define SPR_BULLET_2 4
#define SPR_BULLET_3 5
#define SPR_BULLET_4 6
#define SPR_PLAYERBULLET_1 7
#define SPR_PLAYERBULLET_2 8
#define SPR_ENEMY_1 9
#define SPR_ENEMY_2 10
#define SPR_ENEMY_3 11
#define SPR_ENEMY_4 12

#define TILE_BULLET_BIG_1 0
#define TILE_BULLET_BIG_2 1
#define TILE_PLAYER 2
#define TILE_ENEMY_A 3
#define TILE_ENEMY_B 4
#define TILE_ENEMY_C 5
#define TILE_ENEMY_D 6
#define TILE_HEART 7
#define TILE_BULLET_SMALL_1 8
#define TILE_BULLET_SMALL_2 9
#define TILE_BULLET_SMALL_3 10
#define TILE_BLANK 11

/*
 *
 */

moveable Player;
moveable Bullet[5];
moveable PlayerBullet[2];
moveable Enemies[4];

#include "stages.c"

/*
 * invuln
 */

BYTE invuln_timer = 0;

void invuln_start() {
	invuln_timer = 40;
}

void invuln_process() {
	if (invuln_timer > 0) {
		switch(invuln_timer % 10) {
			case 1:
				set_sprite_tile(SPR_PLAYER, TILE_PLAYER);
				break;
				
			case 5:
				set_sprite_tile(SPR_PLAYER, TILE_BLANK);
				break;
		}
	
		invuln_timer--;
	}
}

/*
 * animation
 */

UBYTE animate_timer = 0;

void animate() {
	animate_timer++;
	
	switch( animate_timer % 20 ) {
		case 1:
			set_sprite_tile(SPR_BULLET_0, TILE_BULLET_BIG_1);
			break;
		
		case 3:
			set_sprite_tile(SPR_BULLET_1, TILE_BULLET_BIG_1);
			break;
			
		case 5:
			set_sprite_tile(SPR_BULLET_2, TILE_BULLET_BIG_1);
			break;
			
		case 7:
			set_sprite_tile(SPR_BULLET_3, TILE_BULLET_BIG_1);
			break;
			
		case 9:
			set_sprite_tile(SPR_BULLET_4, TILE_BULLET_BIG_1);
			break;
			
		case 11:
			set_sprite_tile(SPR_BULLET_0, TILE_BULLET_BIG_2);
			break;
		
		case 13:
			set_sprite_tile(SPR_BULLET_1, TILE_BULLET_BIG_2);
			break;
			
		case 15:
			set_sprite_tile(SPR_BULLET_2, TILE_BULLET_BIG_2);
			break;
			
		case 17:
			set_sprite_tile(SPR_BULLET_3, TILE_BULLET_BIG_2);
			break;
			
		case 19:
			set_sprite_tile(SPR_BULLET_4, TILE_BULLET_BIG_2);
			break;
	}

}

 
/* 
 * player_shoot
 */

void player_shoot() {
	BYTE i;

	for (i = 0; i != 2; ++i) {
		if (! is_visible( &PlayerBullet[i] )) {
			/* SETUP( (&PlayerBullet[i]) , Player.xpos, Player.ypos, 0, -3 ); */ 
			PlayerBullet[i].xpos = PX_TO_FX(50);
			PlayerBullet[i].ypos = PX_TO_FX(150);
			
			copy_position(& PlayerBullet[i], &Player);
			
			PlayerBullet[i].velocity_x = 0;
			PlayerBullet[i].velocity_y = -24;
			break;
		}
	}
}

/*
 * init
 */

void init() {

	set_sprite_data(0, 10, TileLabel);

	/* */
	
	SETUP( (&Player) , PX_TO_FX(84), PX_TO_FX(144), 0, 0 );
	Player.hp = 3;
	Player.sprite_id = SPR_PLAYER;
	set_sprite_tile(SPR_PLAYER, TILE_PLAYER);
	
	/* */
	
	destroy(& Bullet[0]);
	Bullet[0].sprite_id = SPR_BULLET_0;
	set_sprite_tile(SPR_BULLET_0, TILE_BULLET_BIG_1);
	
	destroy(& Bullet[1]);
	Bullet[1].sprite_id = SPR_BULLET_1;
	set_sprite_tile(SPR_BULLET_1, TILE_BULLET_BIG_1);
	
	destroy(& Bullet[2]);
	Bullet[2].sprite_id = SPR_BULLET_2;
	set_sprite_tile(SPR_BULLET_2, TILE_BULLET_BIG_1);
	
	destroy(& Bullet[3]);
	Bullet[3].sprite_id = SPR_BULLET_3;
	set_sprite_tile(SPR_BULLET_3, TILE_BULLET_BIG_1);
	
	destroy(& Bullet[4]);
	Bullet[4].sprite_id = SPR_BULLET_4;
	set_sprite_tile(SPR_BULLET_4, TILE_BULLET_BIG_1);
	
	/* */
	
	destroy(& PlayerBullet[0]);
	PlayerBullet[0].sprite_id = SPR_PLAYERBULLET_1;
	set_sprite_tile(SPR_PLAYERBULLET_1, TILE_BULLET_SMALL_1);
	
	destroy(& PlayerBullet[1]);
	PlayerBullet[1].sprite_id = SPR_PLAYERBULLET_2;
	set_sprite_tile(SPR_PLAYERBULLET_2, TILE_BULLET_SMALL_1);
	
	/* */
		
	destroy(& Enemies[0]);
	Enemies[0].sprite_id = SPR_ENEMY_1;
	destroy(& Enemies[1]);
	Enemies[1].sprite_id = SPR_ENEMY_2;
	destroy(& Enemies[2]);
	Enemies[2].sprite_id = SPR_ENEMY_3;
	destroy(& Enemies[3]);
	Enemies[3].sprite_id = SPR_ENEMY_4;
	
	/* */ 
	
	total_kills = 0;	
	on_kill();
	
	/* */ 
	
	SHOW_SPRITES;
	
}

void update_status_bar() {
	BYTE i;

	for(i = 0; i != 19; ++i) {
		gotoxy( i, 0 );
		setchar(' ');
	}
	
	for (i = 0; i != Player.hp; ++i) {
		gotoxy( 19 - i , 0 );
		setchar(1);
	}
	
}

void hurt_player() {
	invuln_start();
	Player.hp--;
	update_status_bar();
}

void mainloop() {
	UBYTE key;
	
	UBYTE i;
	UBYTE j;
	
	BYTE a_is_down = 0;
	
	while(Player.hp) {
	
		/*wait_vbl_done();*/
		
		animate();
		
		key = joypad();
		if(key & (J_UP|J_DOWN|J_LEFT|J_RIGHT)) {
			if(key & J_UP)
				Player.velocity_y -= 2;
			if(key & J_DOWN)
				Player.velocity_y += 2;
			if(key & J_LEFT)
				Player.velocity_x -= 2;
			if(key & J_RIGHT)
				Player.velocity_x += 2;
		}
		
		if (key & J_A) {
			a_is_down = 1;
		} else if (a_is_down) {
			// keyup
			a_is_down = 0;
			player_shoot();
		}
		
		
		reposition( &Player );
		clamp_to_screen( &Player );
		slowdown( &Player );
		render( &Player );
		
		for (i = 0; i != 2; ++i) {
			reposition( &PlayerBullet[i] );
			destroy_offscreen( &PlayerBullet[i] );
			render( &PlayerBullet[i] );
			
			for (j = 0; j != 5; ++j) {
				if (collides(&PlayerBullet[i], &Bullet[j])) {
					destroy(&PlayerBullet[i]);
					destroy(&Bullet[j]);
				}
			}
			
			for (j = 0; j != 4; ++j) {
				if (collides(&PlayerBullet[i], &Enemies[j])) {
					destroy(&PlayerBullet[i]);
					if (Enemies[j].hp == 0) {
						destroy(&Enemies[j]);
						total_kills++;
						on_kill();
					} else {
						Enemies[j].hp--;
					}
				}
			}
		}
		
		for (i = 0; i != 5; ++i) {
			reposition( &Bullet[i] );
			destroy_offscreen( &Bullet[i] );
			render( &Bullet[i] );
			
			if (!invuln_timer && collides(&Bullet[i], &Player)) {
				destroy(&Bullet[i]);
				hurt_player();
			}
		}
		
		for (i = 0; i != 4; ++i) {
			reposition( &Enemies[i] );
			render( &Enemies[i] );
			
			if (!invuln_timer && collides(&Enemies[i], &Player)) {
				hurt_player();
			}
		}
		
		invuln_process();
		
		plot_progress();
		
	}
}

void clear_all_text() {
	UBYTE x;
	UBYTE y;
	for (x = 0; x < 20; ++x) {
		for(y = 0; y < 20; ++y) {
			gotoxy(x, y);
			setchar(' ');
		}
	}
}

void mainmenu() {
	UBYTE key;
	UWORD last_score;
	
	clear_all_text();
	gotoxy(0, 0);
	
	last_score = 0;
	last_score += total_kills;
	
	printf("GAME\n\nLast Score: %u\n\n      [START]", last_score );
	
	while(1) {
	
		wait_vbl_done();
		key = joypad();
		if (key & J_START) {
			break;
		}
		
	}
	
	clear_all_text();
}

void main() {
	
	while(1) {
		
		mainmenu();
		
		play_garbage();
		
		init();
		
		update_status_bar();
		
		mainloop();
		
		play_garbage(); /* play death sound */
		
	}
}
