
/*
 * moveable
 */

typedef struct {
	BYTE hp;
	BYTE sprite_id;
	WORD xpos;
	WORD ypos;
	BYTE velocity_x;
	BYTE velocity_y;
} moveable;

#define PX_TO_FX(x) ( ((WORD)x) << 3 )
#define FX_TO_PX(x) ( ((WORD)x) >> 3 )

void reposition(moveable* o) {
	o->ypos += o->velocity_y;	
	o->xpos += o->velocity_x;
}

void clamp_to_screen(moveable* o) {
	// top left visible pixel is 8,16

	if (o->ypos < PX_TO_FX(16 )) {
		o->ypos = PX_TO_FX(16 );
		o->velocity_y = 0;
	}
	
	if (o->ypos > PX_TO_FX(152)) {
		o->ypos = PX_TO_FX(152);
		o->velocity_y = 0;
	}

	if (o->xpos < PX_TO_FX(8  )) {
		o->xpos = PX_TO_FX(8  );
		o->velocity_x = 0;
	}
	
	if (o->xpos > PX_TO_FX(160)) {
		o->xpos = PX_TO_FX(160);
		o->velocity_x = 0;
	}
}

void render(moveable* o) {
	move_sprite( o->sprite_id, FX_TO_PX(o->xpos), FX_TO_PX(o->ypos) );
}

void SETUP(moveable* o, WORD x, WORD y, BYTE vx, BYTE vy) {
	o->xpos = x; o->ypos = y; o->velocity_x = vx; o->velocity_y = vy;
}

void destroy(moveable* o) {
	SETUP(o, 0, 0, 0, 0);
}

void destroy_offscreen(moveable* o) {
	if (
		o->ypos && o->xpos &&
		(o->ypos < PX_TO_FX(16) || o->ypos > PX_TO_FX(152) || o->xpos < PX_TO_FX(8) || o->xpos > PX_TO_FX(160))
	) {
		destroy(o);
	}
}

void slowdown(moveable* o) {
	if (o->velocity_y > 0) o->velocity_y--;
	if (o->velocity_y < 0) o->velocity_y++;
	
	if (o->velocity_x > 0) o->velocity_x--;
	if (o->velocity_x < 0) o->velocity_x++;
}

void copy_position(moveable* dest, moveable* src) {
	dest->xpos = src->xpos;
	dest->ypos = src->ypos;
	/* a->velocity_x = b->velocity_x; */ 
}
 
BYTE is_visible(moveable* o) {
	if (o->xpos && o->ypos) {
		return 1;
	}
	return 0;
}
 
#define ABSDIFF(x, y) ( x > y ? x - y : y - x )

BOOLEAN collides(moveable* a, moveable* b) {
	if (
		a->xpos && b->xpos && 
		ABSDIFF(a->xpos, b->xpos) < PX_TO_FX(8-1) &&
		ABSDIFF(a->ypos, b->ypos) < PX_TO_FX(8-1)
	) {
		return 1;
	}
	return 0;
}
